@@ -101,6 +101,15 @@ def _is_builtin(func: collections.abc.Callable):
     return any(func is v for v in vars(builtins).values())
 
 
+def _unwrap(func: typing.Callable):
+    """Unwrap decorated and bounded function."""
+    if hasattr(func, "__func__"):
+        func = func.__func__
+    while hasattr(func, "__wrapped__"):
+        func = func.__wrapped__
+    return func
+
+
 def _get_func_and_arg(
     arg: typing.Optional[str] = None,
     func: typing.Optional[ConfigKey] = None,
@@ -153,9 +162,7 @@ def _get_func_and_arg(
             if len(parent.args) == 0:
                 raise SyntaxError("Partial doesn't have arguments.")
             func = _resolve_func(frame, parent.args[0])
-        # NOTE: `builtins` like `enumerate` are triggered like a class.
-        if inspect.isclass(func) and not _is_builtin(func):
-            func = func.__init__
+        func = _unwrap(func)
 
     _get_func_and_arg_cache[key] = (func, arg)
 
@@ -246,7 +253,7 @@ def purge(usage=True):
     Args:
         usage: Analyze and clear the usage counter instead of persisting it.
     """
-    global _config, _count
+    global _config, _count, _code_to_func
 
     if usage:
         unused = []
@@ -258,6 +265,7 @@ def purge(usage=True):
             warnings.warn("These configurations were not used:\n" + "\n".join(unused))
 
     _config = {}
+    _code_to_func = {}
 
     if usage:
         _count = defaultdict(lambda: defaultdict(int))
@@ -294,7 +302,7 @@ def _check_args(func: ConfigKey, args: ConfigValue):
 
     for key in args.keys():
         if key not in parameters:
-            raise ValueError(f"{key} is not any argument in {func.__qualname__}")
+            raise ValueError(f"`{key}` is not an argument to {func.__qualname__}")
 
 
 def add(config: Config):
@@ -303,16 +311,25 @@ def add(config: Config):
     global _code_to_func
 
     [_check_args(func, args) for func, args in config.items()]
-    _config = {**{k: v.copy() for k, v in config.items()}, **_config}
 
-    functions = [k for k in _config.keys() if not _is_builtin(k)]
-    _code_to_func = {f.__code__: f for f in functions}
-    assert len(_code_to_func) == len(functions), "Invariant error"
+    for key, value in config.items():
+        key = _unwrap(key)
+        if key in _config:
+            _config[key] = Args({**_config[key], **value})
+        else:
+            _config[key] = value.copy()
+
+    funcs = [k for k in _config.keys() if not _is_builtin(k)]
+    _code_to_func = {_unwrap(f.__init__ if inspect.isclass(f) else f).__code__: f for f in funcs}
+    assert len(_code_to_func) == len(funcs), "Invariant error"
 
 
 def partial(func: ConfigKey, *args, **kwargs) -> ConfigKey:
     """Get a `partial` for `func` using the global configuration."""
-    return functools.partial(func, *args, **kwargs, **_config[func])
+    key = _unwrap(func)
+    if key not in _config:
+        raise KeyError(f"`{key.__qualname__}` has not been configured.")
+    return functools.partial(func, *args, **kwargs, **_config[key])
 
 
 def parse_cli_args(args: typing.List[str]) -> Config:
@@ -425,8 +442,17 @@ def profile(frame, event, arg):  # pragma: no cover
         return
 
     function = _code_to_func[frame.f_code]
-    kwargs = _config[function]
-    if not all(frame.f_locals[k] == v for k, v in kwargs.items()):
+    items = _config[function].items()
+    f_locals = frame.f_locals
+    params = inspect.signature(function).parameters
+    var = next((k for k, v in params.items() if v.kind == inspect.Parameter.VAR_KEYWORD), None)
+    if var is None:
+        is_matching = all(f_locals[k] == v for k, v in items)
+    else:
+        kwargs = f_locals[var]
+        is_matching = all((kwargs[k] if k in kwargs else f_locals[k]) == v for k, v in items)
+
+    if not is_matching:
         warnings.warn(
             f"Function `{to_str(function)}` was called at "
             f"({frame.f_back.f_code.co_filename}:{frame.f_back.f_lineno}) with different arguments "
